@@ -15,32 +15,50 @@
 ! You should have received a copy of the GNU Lesser General Public License
 ! along with xtb.  If not, see <https://www.gnu.org/licenses/>.
 
+!> QuickSort algorithm
 recursive subroutine qsort(a, first, last, ind)
-  use xtb_mctc_accuracy, only : wp
-  implicit none
-  real(wp) :: a(*), x, t
-  integer  :: ind(*)
-  integer  :: first, last
-  integer  :: i, j, ii
+   use xtb_mctc_accuracy, only : wp
+   implicit none
+   
+   !> input array
+   real(wp) :: a(*)
+   
+   !> range to sort
+   integer  :: first, last
 
-  x = a( (first+last) / 2 )
-  i = first
-  j = last
-  do
-     do while (a(i) < x)
+   !> index array
+   integer  :: ind(*)
+   
+   !> pivot postion
+   real(wp) :: x
+
+   !> temporary variables
+   integer  :: i, j, ii
+   real(wp) :: t
+
+   x = a( (first+last) / 2 ) ! choose a middle element as pivot
+   i = first
+   j = last
+
+   ! parallel swapping of elements !
+   do
+      do while (a(i) < x)
         i=i+1
-     end do
-     do while (x < a(j))
+      end do
+      do while (x < a(j))
         j=j-1
-     end do
-     if (i >= j) exit
-     t = a(i);  a(i) = a(j);  a(j) = t
-     ii=ind(i); ind(i) = ind(j);  ind(j) = ii
-     i=i+1
-     j=j-1
-  end do
-  if (first < i-1) call qsort(a, first, i-1, ind)
-  if (j+1 < last)  call qsort(a, j+1, last, ind)
+      end do
+      if (i >= j) exit
+      t = a(i);  a(i) = a(j);  a(j) = t
+      ii=ind(i); ind(i) = ind(j);  ind(j) = ii
+      i=i+1
+      j=j-1
+   end do
+
+   ! recursive calls !
+   if (first < i-1) call qsort(a, first, i-1, ind)
+   if (j+1 < last)  call qsort(a, j+1, last, ind)
+
 end subroutine qsort
 
 recursive subroutine qqsort(a, first, last)
