@@ -95,6 +95,12 @@ module xtb_solv_cosmo
       real(wp), allocatable :: dsdr(:, :)
       real(wp), allocatable :: dsdrt(:, :, :)
 
+      !> TM convention? 
+      logical :: tmcosmo = .false.
+      !> Write volume in COSMO file? Volume resolution in Angstrom.
+      logical :: wrvolume = .false.
+      real(wp) :: volumeres = 0.1_wp
+
    contains
 
       !> Update coordinates and internal state
@@ -827,6 +833,7 @@ end subroutine update_nnlist_sasa
 
 !> Write a COSMO file output
 subroutine writeCosmoFile(self, unit, num, sym, xyz, qat, energy)
+   use xtb_solv_ddvolume, only: ddvolume
 
    !> COSMO container
    class(TCosmo), intent(in) :: self
@@ -851,7 +858,7 @@ subroutine writeCosmoFile(self, unit, num, sym, xyz, qat, energy)
 
    integer :: ii, ig, iat
    real(wp) :: dielEnergy, keps
-   real(wp), allocatable :: phi(:), zeta(:), area(:)
+   real(wp), allocatable :: phi(:), zeta(:), area(:), volume(:)
 
    allocate(phi(self%ddCosmo%ncav), zeta(self%ddCosmo%ncav), area(self%ddCosmo%ncav))
    ! Reset potential on the cavity, note that the potential is expected in e/Å
@@ -870,6 +877,10 @@ subroutine writeCosmoFile(self, unit, num, sym, xyz, qat, energy)
       end do
    end do
 
+   !! Switch convention for TM mode
+   if (self%tmcosmo) zeta=-zeta
+   if (self%wrvolume) call ddvolume(xyz,self%rvdw,(self%volumeres)/autoaa,volume)
+
 
     ! Dielectric energy is the energy on the dielectric continuum
    keps = 0.5_wp * (1.0_wp - 1.0_wp/self%dielectricConst)
@@ -886,9 +897,16 @@ subroutine writeCosmoFile(self, unit, num, sym, xyz, qat, energy)
 
    write(unit, '(a)') &
       & "$cosmo_data"
-   write(unit, '(2x, a:, "=", g0)') &
-      & "fepsi", keps, &
-      & "area", sum(area)
+   if (self%wrvolume) then
+      write(unit, '(2x, a:, "=", g0)') &
+         & "fepsi", keps, &
+         & "area", sum(area), &
+         & "volume", sum(volume)
+   else
+      write(unit, '(2x, a:, "=", g0)') &
+         & "fepsi", keps, &
+         & "area", sum(area)
+   end if
 
    write(unit, '(a)') &
       & "$coord_rad", &
@@ -940,6 +958,7 @@ subroutine writeCosmoFile(self, unit, num, sym, xyz, qat, energy)
       & "#", &
       & "#"
 
+
    ii = 0
    do iat = 1, self%ddCosmo%nat
       do ig = 1, self%ddCosmo%ngrid
